package conferenciaBDAlunos;

import java.util.ArrayList;
import java.util.List;

public class Artigo {

	private String nome;
	private String resumo;
	private int identificador;
	private String nomeArquivo;
	private double notaFinal;
	private int status;
	private boolean temRevisao;

	private List<Autor> autores;
	private List<Avaliador> avaliadoresConflitantes; // tem conflito com
	// avaliadores
	private List<Topico> topicos;
	private List<Revisao> revisoes;

	public Artigo(String nome) {
		this.nome = nome;
		status = 0; // por default e' nao aceito
		autores = new ArrayList<Autor>();
		avaliadoresConflitantes = new ArrayList<Avaliador>();
		revisoes = new ArrayList<Revisao>();
		topicos = new ArrayList<Topico>();
		temRevisao = false;
	}

	public int getIdentificador() {
		return identificador;
	}

	public String getNome() {
		return nome;
	}

	public String getNomeArquivo() {
		return nomeArquivo;
	}

	public double getNotaFinal() {
		return notaFinal;
	}

	public String getResumo() {
		return resumo;
	}

	public int getStatus() {
		return status;
	}

	public void setIdentificador(int identificador) {
		this.identificador = identificador;
	}

	public void setNome(String nome) {
		this.nome = nome;
	}

	public void setNomeArquivo(String nomeArquivo) {
		this.nomeArquivo = nomeArquivo;
	}

	public void setNotaFinal(double notaFinal) {
		this.notaFinal = notaFinal;
	}

	public void setResumo(String resumo) {
		this.resumo = resumo;
	}

	public void setStatus(int status) {
		this.status = status;
	}

	public void setTemRevisao(boolean temRevisao) {
		this.temRevisao = temRevisao;
	}

	public void associaAutor(Autor autor) {
		if (!autores.contains(autor))
			autores.add(autor);
	}

	public void associaTopico(Topico topico) {
		if (!topicos.contains(topico))
			topicos.add(topico);
	}

	public void defineConflito(Avaliador avaliador) {
		if (!avaliadoresConflitantes.contains(avaliador))
			avaliadoresConflitantes.add(avaliador);
	}

	public void associaRevisao(Revisao revisao) {
		temRevisao = true;
		if (revisoes.size() < 3 && !revisoes.contains(revisao))
			revisoes.add(revisao);
	}

	public void calculaNotaFinal() {
		double soma = 0;
		int quantas_revisoes = revisoes.size();
		for (int i = 0; i < revisoes.size(); i++)
			soma = soma + revisoes.get(i).getNotaFinal();
		notaFinal = quantas_revisoes > 0 ? soma / quantas_revisoes : 0;
	}

	public void defineComoAceito() {
		status = 1;
	}

	public void modificaStatus() {
		if (status == 1)
			status = 0;
		else
			status = 1;
	}

	public boolean isTemRevisao() {
		return temRevisao;
	}

	public List<Revisao> getRevisoes() {
		return revisoes;
	}

	public List<Autor> getAutores() {
		return autores;
	}

	public List<Topico> getTopicos() {
		return topicos;
	}

	public List<Avaliador> getRevisoresComConflito() {
		return avaliadoresConflitantes;
	}

	public boolean equals(Object obj) {
		if (obj instanceof Artigo)
			return nome.equals(((Artigo) obj).nome);
		else
			return false;
	}

	public String toString() {
		return nome + " " + identificador + " " + status;
	}

}
