package conferenciaBDAlunos;

import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class Conferencia {

	private static Conferencia conferencia;

	private Connection con;
	private boolean iniciada;

	private String nome;
	private String local;
	private Date data;
	private int numMaxArtigos;
	private Date dataInicioSubmissao;
	private Date dataTerminoSubmissao;
	private Date dataTerminoRevisao;

	public static Conferencia getInstance() {
		if (conferencia == null)
			conferencia = new Conferencia();
		return conferencia;
	}

	private Conferencia() {

		try {
			// System.out.println(new File(".").getAbsolutePath());

			// BD MySQL
			Class.forName("com.mysql.jdbc.Driver");
			con = DriverManager
					.getConnection("jdbc:mysql://localhost/conferenciaBD",
							"root", "ufliictsc");
			// DriverManager.getConnection(
			// "jdbc:mysql://localhost/test?user=monty&password=greatsqldb");

			// BD hsqldb
			// Class.forName("org.hsqldb.jdbcDriver");
			// this.con =
			// DriverManager.getConnection("jdbc:hsqldb:BD"+File.separator
			// +"conferenciaBD","sa","");

			iniciada = MapeadorConferencia.getInstance().get(con, this) != null;
		} catch (Exception e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao conectar");
		}
	}

	public String getNome() {
		return nome;
	}

	public Date getData() {
		return data;
	}

	public Date getDataInicioSubmissao() {
		return dataInicioSubmissao;
	}

	public Date getDataTerminoRevisao() {
		return dataTerminoRevisao;
	}

	public Date getDataTerminoSubmissao() {
		return dataTerminoSubmissao;
	}

	public String getLocal() {
		return local;
	}

	public int getNumMaxArtigos() {
		return numMaxArtigos;
	}

	public void setData(Date data) {
		this.data = data;
	}

	public void setDataInicioSubmissao(Date dataInicioSubmissao) {
		this.dataInicioSubmissao = dataInicioSubmissao;
	}

	public void setDataTerminoRevisao(Date dataTerminoRevisao) {
		this.dataTerminoRevisao = dataTerminoRevisao;
	}

	public void setDataTerminoSubmissao(Date dataTerminoSubmissao) {
		this.dataTerminoSubmissao = dataTerminoSubmissao;
	}

	public void setLocal(String local) {
		this.local = local;
	}

	public void setNome(String nome) {
		this.nome = nome;
	}

	public void setNumMaxArtigos(int numMaxArtigos) {
		this.numMaxArtigos = numMaxArtigos;
	}

	public void iniciaConferencia(String nome, String local, Date data,
			int numMaxArtigos, Date dataInicioSubmissao,
			Date dataTerminoSubmissao, Date dataTerminoRevisao,
			String[] nomesTopicos) {
		if (!isIniciada()) {
			this.nome = nome;
			this.local = local;
			this.data = data;
			this.numMaxArtigos = numMaxArtigos;
			this.dataInicioSubmissao = dataInicioSubmissao;
			this.dataTerminoSubmissao = dataTerminoSubmissao;
			this.dataTerminoRevisao = dataTerminoRevisao;
			try {
				MapeadorConferencia.getInstance().put(con, this);
				iniciada = true;
				for (int i = 0; i < nomesTopicos.length; i++) {
					Topico topico = new Topico();
					topico.setNome(nomesTopicos[i]);
					MapeadorTopico.getInstance().put(con, topico);
				}
			} catch (SQLException e) {
				e.printStackTrace();
				throw new java.lang.RuntimeException(
						"erro ao inserir conferencia");
			}
		}
	}

	boolean isIniciada() {
		return iniciada;
	}

	public void insereTopico(String nome) {
		Topico topico = new Topico();
		topico.setNome(nome);
		try {
			MapeadorTopico.getInstance().put(con, topico);
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao inserir topico");
		}
	}

	public void defineChefe(String nome, String login, String senha) {
		try {

			ChefeComite.inicializaClasseChefeComite(nome);
			ChefeComite chefe = MapeadorChefeComite.getInstance()
					.get(con, nome);
			if (chefe == null) {
				chefe = ChefeComite.getInstance();
				chefe.setLogin(login);
				chefe.setSenha(senha);
				MapeadorChefeComite.getInstance().put(con, chefe);
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao inserir chefe");
		}
	}

	public void insereAvaliador(String nome, String login, String senha,
			String email, String origem, String[] nomesTopicos) {
		Avaliador avaliador = new Avaliador(nome, login, senha, email, origem);
		for (int i = 0; i < nomesTopicos.length; i++) {
			String nomeTopico = nomesTopicos[i];
			try {
				Topico topico = MapeadorTopico.getInstance().get(con,
						nomeTopico);
				if (topico != null)
					avaliador.associaTopico(topico);
			} catch (SQLException e) {
				e.printStackTrace();
				throw new java.lang.RuntimeException("erro ao associar topico");
			}
		}
		try {
			MapeadorAvaliador.getInstance().put(con, avaliador);
			// Enviar email para o avaliador
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao inserir avaliador");
		}
	}

	public void mandaEmail() {
	}

	private int nextIdentificador() throws SQLException {
		PreparedStatement stmt = con
				.prepareStatement("SELECT MAX(IDENTIFICADOR) FROM ARTIGO");
		ResultSet rs = stmt.executeQuery();
		try {
			if (rs.next()) {
				Object ident = rs.getObject(1);
				if (ident != null)
					return rs.getInt(1) + 1;
			}
			return 1;
		} finally {
			rs.close();
			stmt.close();
		}
	}

	public void submeteArtigo(String nome, String resumo, String nomeArquivo,
			String[] nomesAutores, String[] instituicoesAutores,
			String[] nomesTopicos) {
		Artigo artigo;
		try {
			artigo = MapeadorArtigo.getInstance().get(con, nome);
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao carregar artigo");
		}
		if (artigo == null) {
			artigo = new Artigo(nome);
			artigo.setResumo(resumo);
			try {
				artigo.setIdentificador(nextIdentificador());
			} catch (SQLException e) {
				e.printStackTrace();
				throw new java.lang.RuntimeException(
						"erro ao obter identificador");
			}
			artigo.setNomeArquivo(nomeArquivo);
			for (int i = 0; i < nomesAutores.length; i++) {
				String nomeAutor = nomesAutores[i];
				String instituicaoAutor = instituicoesAutores[i];
				try {
					Autor autor = MapeadorAutor.getInstance().get(con,
							nomeAutor);
					if (autor == null) {
						autor = new Autor(nomeAutor, instituicaoAutor);
						MapeadorAutor.getInstance().put(con, autor);
					}
					artigo.associaAutor(autor);
				} catch (SQLException e) {
					e.printStackTrace();
					throw new java.lang.RuntimeException(
							"erro ao associar autor");
				}
			}

			for (int i = 0; i < nomesTopicos.length; i++) {
				String nomeTopico = nomesTopicos[i];
				try {
					Topico topico = MapeadorTopico.getInstance().get(con,
							nomeTopico);
					if (topico != null)
						artigo.associaTopico(topico);
				} catch (SQLException e) {
					e.printStackTrace();
					throw new java.lang.RuntimeException(
							"erro ao associar topico");
				}
			}
			try {
				MapeadorArtigo.getInstance().put(con, artigo);
			} catch (SQLException e) {
				e.printStackTrace();
				throw new java.lang.RuntimeException("erro ao inserir artigo");
			}
		}
	}

	public List<Artigo> selecionaArtigosSemRevisao() {
		// Retorna um vetor contendo os artigos que ainda nao possuem nenhuma
		// revisao
		try {
			return MapeadorArtigo.getInstance().selectArtigosSemRevisao(con);
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao selecionar artigos");
		}
	}

	public List<Avaliador> selecionaAvaliadoresSemConflito(String nomeArtigo) {
		// Retorna um vetor contendo os avaliadores sem conflito com um artigo.
		// \
		try {
			Artigo artigo = MapeadorArtigo.getInstance().get(con, nomeArtigo);
			List<Avaliador> avaliadoresConflitantes = artigo
					.getRevisoresComConflito();
			List<Avaliador> avaliadores = MapeadorAvaliador.getInstance()
					.selectTodos(con);
			List<Avaliador> avaliadoresSemConflitos = new ArrayList<Avaliador>();
			for (int i = 0; i < avaliadores.size(); i++) {
				Avaliador avaliador = avaliadores.get(i);
				List<Artigo> artigos = avaliador.retornaArtigosAvaliados();
				if (!artigos.contains(artigo)
						&& !avaliadoresConflitantes.contains(avaliador))
					avaliadoresSemConflitos.add(avaliador);
			}
			return avaliadoresSemConflitos;
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException(
					"erro ao selecionar os avaliadores sem conflito");
		}
	}

	public void defineConflito(String nomeArtigo, String nomeAvaliador) {
		try {
			Artigo artigo = MapeadorArtigo.getInstance().get(con, nomeArtigo);
			Avaliador avaliador = MapeadorAvaliador.getInstance().get(con,
					nomeAvaliador);
			artigo.defineConflito(avaliador);
			MapeadorArtigo.getInstance().put(con, artigo);
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao definir conflito");
		}
	}

	public void distribuiArtigos() {
		List<Avaliador> avaliadores = null;
		List<Artigo> artigos = null;

		try {
			avaliadores = MapeadorAvaliador.getInstance().selectTodos(con);
			artigos = MapeadorArtigo.getInstance().selectTodos(con);
		} catch (SQLException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		int numAvaliadores = avaliadores.size();
		int contAvaliadores = 0;
		for (int art = 0; art < artigos.size(); art++) {
			Artigo artigo = artigos.get(art);
			for (int i = 1; i <= 3; i++) {
				int j = 1;
				boolean encontrou = false;
				while (j <= numAvaliadores && !encontrou) {
					Avaliador avaliador = avaliadores.get(contAvaliadores);
					if (!artigo.getRevisoresComConflito().contains(avaliador)) {
						List<Revisao> revisoes = artigo.getRevisoes();
						boolean avalia = false;
						for (int k = 0; k < revisoes.size(); k++)
							if (revisoes.get(k).getAvaliador()
									.equals(avaliador))
								avalia = true;
						if (!avalia) {
							encontrou = true;
							Revisao revisao = new Revisao();
							revisao.setArtigo(artigo);
							revisao.setAvaliador(avaliador);
							artigo.associaRevisao(revisao);
							avaliador.associaRevisao(revisao);
							try {
								MapeadorArtigo.getInstance().put(con, artigo);
								MapeadorAvaliador.getInstance().put(con,
										avaliador);
								MapeadorRevisao.getInstance().put(con, revisao);

							} catch (SQLException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}

						}
					}
					j++;
					contAvaliadores++;
					if (contAvaliadores == numAvaliadores)
						contAvaliadores = 0;
				}
			}
		}
	}

	public Artigo selecionaArtigo(String nome) {
		try {
			return MapeadorArtigo.getInstance().get(con, nome);
		} catch (SQLException e) {
			return null;
		}
	}

	public void forneceRevisao(String nomeArtigo, String nomeAvaliador,
			double clareza, double relevancia, double corretude,
			double notaFinal, String comentariosComite, String comentariosAutor) {
		try {
			Avaliador avaliador = MapeadorAvaliador.getInstance().get(con,
					nomeAvaliador);

			// List<Artigo> artigosDesteAvaliador =
			// avaliador.retornaArtigosAvaliados();

			Artigo artigo = MapeadorArtigo.getInstance().get(con, nomeArtigo);
			List<Revisao> revisoes = artigo.getRevisoes();
			int revisoesDesteAvaliador = 0;
			for (int i = 0; i < revisoes.size(); i++) {
				Revisao revisao = revisoes.get(i);
				if (revisao.getAvaliador().equals(avaliador)) {
					revisoesDesteAvaliador++;
					revisao.setRevisaoJahExistente(clareza, relevancia,
							corretude, notaFinal, comentariosComite,
							comentariosAutor);
				}
			}

			if (revisoesDesteAvaliador == 0) {
				Revisao novaRevisao = new Revisao();
				novaRevisao.setAvaliador(avaliador);
				novaRevisao.setArtigo(artigo);
				MapeadorRevisao.getInstance().put(con, novaRevisao);
			}

		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao fornecer revisao");
		}
	}

	public List<Artigo> selecionaArtigos() {
		// Seleciona todos os artigos
		try {
			return MapeadorArtigo.getInstance().selectTodos(con);
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao selecionar artigos");
		}
	}

	public List<Avaliador> selecionaAvaliadores() {
		// Seleciona todos os avaliadores
		try {
			return MapeadorAvaliador.getInstance().selectTodos(con);
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException(
					"erro ao selecionar avaliadores");
		}
	}

	public List<Artigo> selecionaArtigosComNotas() {
		try {
			List<Artigo> artigos = selecionaArtigos();
			for (int i = 0; i < artigos.size(); i++) {
				Artigo artigo = artigos.get(i);
				artigo.calculaNotaFinal();
				MapeadorArtigo.getInstance().put(con, artigo);
			}
			return artigos;
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao selecionar artigos");
		}

	}

	public void defineArtigosAceitos(String[] nomesArtigos) {
		try {
			for (int i = 0; i < nomesArtigos.length; i++) {
				Artigo artigo = MapeadorArtigo.getInstance().get(con,
						nomesArtigos[i]);
				artigo.defineComoAceito();
				MapeadorArtigo.getInstance().put(con, artigo);
			}
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao selecionar artigos");
		}
	}

	public void modificaStatusArtigo(String nomeArtigo) {
		try {
			Artigo artigo = MapeadorArtigo.getInstance().get(con, nomeArtigo);
			artigo.modificaStatus();
			MapeadorArtigo.getInstance().put(con, artigo);
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao selecionar artigo");
		}
	}

	public void termina() {
		try {
			if (!con.isClosed())
				con.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao fechar conexao");
		}
	}

}
