package conferenciaBDAlunos;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

public class MapeadorAutor {

	private static MapeadorAutor mapeador;

	private Map<String, Autor> cache;

	private MapeadorAutor() {
		super();
		cache = new HashMap<String, Autor>();
	}

	public static MapeadorAutor getInstance() {
		if (mapeador == null)
			mapeador = new MapeadorAutor();
		return mapeador;
	}

	public Autor get(Connection con, String nome) throws SQLException {
		Autor autor = cache.get(nome);
		if (autor == null) {
			PreparedStatement stmt = con
					.prepareStatement("SELECT INSTITUICAO FROM PESSOA "
							+ "WHERE NOME=? AND TIPO=3");
			stmt.setString(1, nome);
			ResultSet rs = stmt.executeQuery();
			try {
				if (rs.next()) {
					autor = new Autor(nome, rs.getString("INSTITUICAO"));
					cache.put(nome, autor);
					return autor;
				} else
					return null;
			} finally {
				rs.close();
				stmt.close();
			}
		} else
			return autor;
	}

	public void put(Connection con, Autor aut) throws SQLException {
		if (get(con, aut.getNome()) != null) {
			PreparedStatement stmt = con.prepareStatement("UPDATE AUTOR SET "
					+ "INSTITUICAO=? WHERE NOME=? AND TIPO=3");
			try {
				stmt.setString(1, aut.getInstituicao());
				stmt.setString(2, aut.getNome());
				int updtCnt = stmt.executeUpdate();
				if (updtCnt == 0)
					throw new SQLException("Nome inexistente!");
			} finally {
				stmt.close();
			}
		} else {
			PreparedStatement stmt = con
					.prepareStatement("INSERT INTO PESSOA (NOME,INSTITUICAO,TIPO) "
							+ "VALUES (?,?,3)");
			try {
				stmt.setString(1, aut.getNome());
				stmt.setString(2, aut.getInstituicao());
				stmt.execute();
			} finally {
				stmt.close();
			}
		}
		cache.put(aut.getNome(), aut);

	}

}
