package conferenciaBDAlunos;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MapeadorAvaliador {

	private static MapeadorAvaliador mapeador;

	private Map<String, Avaliador> cache;

	private MapeadorAvaliador() {
		super();
		cache = new HashMap<String, Avaliador>();
	}

	public static MapeadorAvaliador getInstance() {
		if (mapeador == null)
			mapeador = new MapeadorAvaliador();
		return mapeador;
	}

	public Avaliador get(Connection con, String nome) throws SQLException {
		Avaliador aval = cache.get(nome);
		if (aval == null) {
			PreparedStatement stmt = con
					.prepareStatement("SELECT LOGIN,SENHA,EMAIL,ORIGEM FROM PESSOA "
							+ "WHERE NOME=? AND TIPO=2");
			stmt.setString(1, nome);
			ResultSet rs = stmt.executeQuery();
			try {
				if (rs.next()) {
					aval = new Avaliador(nome, rs.getString("LOGIN"), rs
							.getString("SENHA"), rs.getString("EMAIL"), rs
							.getString("ORIGEM"));
					cache.put(nome, aval);
					getTopicos(con, aval);
					getRevisoes(con, aval);
					return aval;
				} else
					return null;
			} finally {
				rs.close();
				stmt.close();
			}
		} else
			return aval;
	}

	private void getTopicos(Connection con, Avaliador aval) throws SQLException {
		PreparedStatement stmt = con
				.prepareStatement("SELECT NOME_TOPICO FROM TOPICO_AVALIADOR "
						+ "WHERE NOME_AVALIADOR=?");
		stmt.setString(1, aval.getNome());
		ResultSet rs = stmt.executeQuery();
		try {
			while (rs.next()) {
				Topico top = MapeadorTopico.getInstance().get(con,
						rs.getString("NOME_TOPICO"));
				aval.associaTopico(top);
			}
		} finally {
			rs.close();
			stmt.close();
		}
	}

	private void getRevisoes(Connection con, Avaliador aval)
			throws SQLException {
		PreparedStatement stmt = con
				.prepareStatement("SELECT NOME_ARTIGO,NOME_AVALIADOR FROM REVISAO    "
						+ "WHERE NOME_AVALIADOR=?");
		stmt.setString(1, aval.getNome());
		ResultSet rs = stmt.executeQuery();
		try {
			while (rs.next()) {
				Revisao rev = MapeadorRevisao.getInstance().get(con,
						rs.getString("NOME_ARTIGO"),
						rs.getString("NOME_AVALIADOR"));
				aval.associaRevisao(rev);
			}
		} finally {
			rs.close();
			stmt.close();
		}
	}

	private void putTopicos(Connection con, Avaliador aval) throws SQLException {
		// atualiza as associacoes com os topicos na base de dados
		PreparedStatement stmt = con
				.prepareStatement("DELETE FROM TOPICO_AVALIADOR WHERE NOME_AVALIADOR=?");
		try {
			stmt.setString(1, aval.getNome());
			stmt.executeUpdate();
			List<Topico> topicos = aval.getTopicos();
			for (int i = 0; i < topicos.size(); i++) {
				Topico topico = topicos.get(i);
				stmt = con
						.prepareStatement("INSERT INTO TOPICO_AVALIADOR (NOME_AVALIADOR,NOME_TOPICO) "
								+ "VALUES (?,?)");
				stmt.setString(1, aval.getNome());
				stmt.setString(2, topico.getNome());
			}
		} finally {
			stmt.close();
		}
	}

	public void put(Connection con, Avaliador aval) throws SQLException {
		if (get(con, aval.getNome()) != null) {
			PreparedStatement stmt = con
					.prepareStatement("UPDATE PESSOA SET "
							+ "LOGIN=?,SENHA=?,EMAIL=?,ORIGEM=? WHERE NOME=? AND TIPO=2");
			try {
				stmt.setString(1, aval.getLogin());
				stmt.setString(2, aval.getSenha());
				stmt.setString(3, aval.getEmail());
				stmt.setString(4, aval.getOrigem());
				stmt.setString(5, aval.getNome());
				int updtCnt = stmt.executeUpdate();
				if (updtCnt == 0)
					throw new SQLException("Nome inexistente!");
			} finally {
				stmt.close();
			}
			putTopicos(con, aval);
		} else {
			PreparedStatement stmt = con
					.prepareStatement("INSERT INTO PESSOA (NOME,LOGIN,SENHA,EMAIL,ORIGEM,TIPO) "
							+ "VALUES (?,?,?,?,?,2)");
			try {
				stmt.setString(1, aval.getNome());
				stmt.setString(2, aval.getLogin());
				stmt.setString(3, aval.getSenha());
				stmt.setString(4, aval.getEmail());
				stmt.setString(5, aval.getOrigem());
				stmt.execute();
			} finally {
				stmt.close();
			}
			putTopicos(con, aval);
		}
		cache.put(aval.getNome(), aval);
	}

	public List<Avaliador> selectTodos(Connection con) throws SQLException {
		PreparedStatement stmt = con
				.prepareStatement("SELECT NOME,LOGIN,SENHA,EMAIL,ORIGEM FROM PESSOA WHERE TIPO=2");
		ResultSet rs = stmt.executeQuery();
		List<Avaliador> avaliadores = new ArrayList<Avaliador>();
		try {
			while (rs.next()) {
				Avaliador aval = new Avaliador(rs.getString("NOME"), rs
						.getString("LOGIN"), rs.getString("SENHA"), rs
						.getString("EMAIL"), rs.getString("ORIGEM"));
				cache.put(aval.getNome(), aval);
				getTopicos(con, aval);
				avaliadores.add(aval);
			}
		} finally {
			rs.close();
			stmt.close();
		}
		return avaliadores;
	}

}
