package conferenciaBDAlunos;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

public class MapeadorChefeComite {

	private static MapeadorChefeComite mapeador;

	private Map<String, ChefeComite> cache;

	private MapeadorChefeComite() {
		super();
		cache = new HashMap<String, ChefeComite>();
	}

	public static MapeadorChefeComite getInstance() {
		if (mapeador == null)
			mapeador = new MapeadorChefeComite();
		return mapeador;
	}

	public ChefeComite get(Connection con, String nome) throws SQLException {
		ChefeComite chefe = cache.get(nome);
		if (chefe == null) {
			PreparedStatement stmt = con
					.prepareStatement("SELECT LOGIN,SENHA FROM PESSOA "
							+ "WHERE NOME = ? AND TIPO=1");
			stmt.setString(1, nome);
			ResultSet rs = stmt.executeQuery();
			try {
				if (rs.next()) {
					ChefeComite.inicializaClasseChefeComite(nome);
					chefe = ChefeComite.getInstance();
					chefe.setLogin(rs.getString("LOGIN"));
					chefe.setSenha(rs.getString("SENHA"));
					cache.put(nome, chefe);
					return chefe;
				} else
					return null;
			} finally {
				rs.close();
				stmt.close();
			}
		} else
			return chefe;
	}

	public void put(Connection con, ChefeComite chefe) throws SQLException {
		if (get(con, chefe.getNome()) != null) {
			PreparedStatement stmt = con.prepareStatement("UPDATE PESSOA SET "
					+ "LOGIN=?,SENHA=? WHERE NOME=? AND TIPO=1");
			try {
				stmt.setString(1, chefe.getLogin());
				stmt.setString(2, chefe.getSenha());
				stmt.setString(3, chefe.getNome());
				int updtCnt = stmt.executeUpdate();
				if (updtCnt == 0)
					throw new SQLException("Nome inexistente!");
			} finally {
				stmt.close();
			}
		} else {
			PreparedStatement stmt = con
					.prepareStatement("INSERT INTO PESSOA (NOME,LOGIN,SENHA,TIPO) "
							+ "VALUES (?,?,?,1)");
			try {
				stmt.setString(1, chefe.getNome());
				stmt.setString(2, chefe.getLogin());
				stmt.setString(3, chefe.getSenha());
				stmt.execute();
			} finally {
				stmt.close();
			}
		}
		cache.put(chefe.getNome(), chefe);
	}

}
