package conferenciaBDAlunos;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

public class MapeadorConferencia {

	private static MapeadorConferencia mapeador;

	private MapeadorConferencia() {
		super();
	}

	public static MapeadorConferencia getInstance() {
		if (mapeador == null)
			mapeador = new MapeadorConferencia();
		return mapeador;
	}

	public Conferencia get(Connection con, Conferencia confer)
			throws SQLException {
		if (confer.isIniciada())
			return confer;
		else {
			PreparedStatement stmt = con
					.prepareStatement("SELECT NOME,LOCAL,DATA,MAX_ARTIGOS,INICIO_SUB,"
							+ "TERMINO_SUB,TERMINO_REV FROM CONFERENCIA ");
			ResultSet rs = stmt.executeQuery();
			try {
				if (rs.next()) {
					confer.setNome(rs.getString("NOME"));
					confer.setLocal(rs.getString("LOCAL"));
					confer.setData(rs.getDate("DATA"));
					confer.setNumMaxArtigos(rs.getInt("MAX_ARTIGOS"));
					confer.setDataInicioSubmissao(rs.getDate("INICIO_SUB"));
					confer.setDataTerminoSubmissao(rs.getDate("TERMINO_SUB"));
					confer.setDataTerminoRevisao(rs.getDate("TERMINO_REV"));
					List<Topico> topicos = MapeadorTopico.getInstance()
							.selectTodos(con);
					for (int i = 0; i < topicos.size(); i++) {
						String nomeTopico = topicos.get(i).getNome();
						confer.insereTopico(nomeTopico);
					}
					return confer;
				}
				return null;
			} finally {
				rs.close();
				stmt.close();
			}
		}
	}

	public void put(Connection con, Conferencia conf) throws SQLException {
		if (get(con, conf) != null) {
			PreparedStatement stmt = con
					.prepareStatement("UPDATE CONFERENCIA SET "
							+ "LOCAL=?,DATA=?,MAX_ARTIGOS=?,INICIO_SUB=?,TERMINO_SUB=?,TERMINO_REV=?"
							+ " WHERE NOME=?");
			try {
				stmt.setString(1, conf.getLocal());
				stmt.setDate(2, conf.getData());
				stmt.setInt(3, conf.getNumMaxArtigos());
				stmt.setDate(4, conf.getDataInicioSubmissao());
				stmt.setDate(5, conf.getDataTerminoSubmissao());
				stmt.setDate(6, conf.getDataTerminoRevisao());
				stmt.setString(7, conf.getNome());
				int updtCnt = stmt.executeUpdate();
				if (updtCnt == 0)
					throw new SQLException("Nome inexistente!");
			} finally {
				stmt.close();
			}
		} else {
			PreparedStatement stmt = con
					.prepareStatement("INSERT INTO CONFERENCIA (NOME,LOCAL,DATA,"
							+ "MAX_ARTIGOS,INICIO_SUB,TERMINO_SUB,TERMINO_REV) VALUES (?,?,?,?,?,?,?)");
			try {
				stmt.setString(1, conf.getNome());
				stmt.setString(2, conf.getLocal());
				stmt.setDate(3, conf.getData());
				stmt.setInt(4, conf.getNumMaxArtigos());
				stmt.setDate(5, conf.getDataInicioSubmissao());
				stmt.setDate(6, conf.getDataTerminoSubmissao());
				stmt.setDate(7, conf.getDataTerminoRevisao());
				stmt.execute();
			} finally {
				stmt.close();
			}
		}
	}
}
