package conferenciaBDAlunos;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

public class MapeadorRevisao {

	private static MapeadorRevisao mapeador;

	private Map<String, Revisao> cache;

	private MapeadorRevisao() {
		super();
		cache = new HashMap<String, Revisao>();
	}

	public static MapeadorRevisao getInstance() {
		if (mapeador == null)
			mapeador = new MapeadorRevisao();
		return mapeador;
	}

	public Object get(Connection con, Object chave) throws SQLException {
		if (chave instanceof Object[] && ((Object[]) chave).length == 2) {
			String nomeArtigo = ((Object[]) chave)[0].toString();
			String nomeAvaliador = ((Object[]) chave)[1].toString();
			return this.get(con, nomeArtigo, nomeAvaliador);
		} else
			throw new IllegalArgumentException("chave de tipo invalido");
	}

	public Revisao get(Connection con, String nomeArtigo, String nomeAvaliador)
			throws SQLException {
		Revisao rev = cache.get(nomeArtigo + nomeAvaliador);
		if (rev == null) {
			PreparedStatement stmt = con
					.prepareStatement("SELECT CLAREZA,RELEVANCIA,CORRETUDE,NOTA_FINAL,"
							+ "COMENT_COMITE,COMENT_AUTOR FROM REVISAO WHERE NOME_ARTIGO=? AND NOME_AVALIADOR=?");
			stmt.setString(1, nomeArtigo);
			stmt.setString(2, nomeAvaliador);
			ResultSet rs = stmt.executeQuery();
			try {
				if (rs.next()) {
					rev = new Revisao();
					rev.setClareza(rs.getDouble("CLAREZA"));
					rev.setRelevancia(rs.getDouble("RELEVANCIA"));
					rev.setCorretude(rs.getDouble("CORRETUDE"));
					rev.setNotaFinal(rs.getDouble("NOTA_FINAL"));
					rev.setComentariosComite(rs.getString("COMENT_COMITE"));
					rev.setComentariosAutor(rs.getString("COMENT_AUTOR"));
					cache.put(nomeArtigo + nomeAvaliador, rev);
					rev.associaArtigo(MapeadorArtigo.getInstance().get(con,
							nomeArtigo));
					rev.associaAvaliador(MapeadorAvaliador.getInstance().get(
							con, nomeAvaliador));
					return rev;
				} else
					return null;
			} finally {
				rs.close();
				stmt.close();
			}
		} else
			return rev;
	}

	public void put(Connection con, Revisao rev) throws SQLException {
		if (this.get(con, rev.getArtigo().getNome(), rev.getAvaliador()
				.getNome()) != null) {
			PreparedStatement stmt = con
					.prepareStatement("UPDATE REVISAO SET "
							+ "CLAREZA=?,RELEVANCIA=?,CORRETUDE=?,NOTA_FINAL=?,COMENT_COMITE=?,COMENT_AUTOR=? WHERE "
							+ "NOME_ARTIGO=? AND NOME_AVALIADOR=?");
			try {
				stmt.setDouble(1, rev.getClareza());
				stmt.setDouble(2, rev.getRelevancia());
				stmt.setDouble(3, rev.getCorretude());
				stmt.setDouble(4, rev.getNotaFinal());
				stmt.setString(5, rev.getComentariosComite());
				stmt.setString(6, rev.getComentariosAutor());
				stmt.setString(7, rev.getArtigo().getNome());
				stmt.setString(8, rev.getAvaliador().getNome());
				int updtCnt = stmt.executeUpdate();
				if (updtCnt == 0)
					throw new SQLException("Nomes inexistentes!");
			} finally {
				stmt.close();
			}
		} else {
			PreparedStatement stmt = con
					.prepareStatement("INSERT INTO REVISAO (NOME_ARTIGO,NOME_AVALIADOR,CLAREZA,"
							+ "RELEVANCIA,CORRETUDE,NOTA_FINAL,COMENT_COMITE,COMENT_AUTOR) VALUES (?,?,?,?,?,?,?,?)");
			try {
				stmt.setString(1, rev.getArtigo().getNome());
				stmt.setString(2, rev.getAvaliador().getNome());
				stmt.setDouble(3, rev.getClareza());
				stmt.setDouble(4, rev.getRelevancia());
				stmt.setDouble(5, rev.getCorretude());
				stmt.setDouble(6, rev.getNotaFinal());
				stmt.setString(7, rev.getComentariosComite());
				stmt.setString(8, rev.getComentariosAutor());
				stmt.execute();

				rev.getArtigo().associaRevisao(rev);
				stmt = con
						.prepareStatement("UPDATE ARTIGO SET TEM_REVISAO=? WHERE NOME=?");
				stmt.setBoolean(1, true);
				stmt.setString(2, rev.getArtigo().getNome());

			} finally {
				stmt.close();
			}
		}
		cache
				.put(rev.getArtigo().getNome() + rev.getAvaliador().getNome(),
						rev);
	}

}
