package conferenciaBDAlunos;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MapeadorTopico {

	private static MapeadorTopico mapeador;

	private Map<String, Topico> cache;

	private MapeadorTopico() {
		super();
		cache = new HashMap<String, Topico>();
	}

	public static MapeadorTopico getInstance() {
		if (mapeador == null)
			mapeador = new MapeadorTopico();
		return mapeador;
	}

	public Topico get(Connection con, String nome) throws SQLException {
		Topico top = cache.get(nome);
		if (top == null) {
			PreparedStatement stmt = con
					.prepareStatement("SELECT NOME FROM TOPICO WHERE NOME = ?");
			stmt.setString(1, nome);
			ResultSet rs = stmt.executeQuery();
			try {
				if (rs.next()) {
					top = new Topico();
					top.setNome(nome);
					cache.put(nome, top);
					return top;
				} else
					return null;
			} finally {
				rs.close();
				stmt.close();
			}
		} else
			return top;
	}

	public void put(Connection con, Topico top) throws SQLException {
		if (get(con, top.getNome()) == null) {
			PreparedStatement stmt = con
					.prepareStatement("INSERT INTO TOPICO (NOME) VALUES (?)");
			try {
				stmt.setString(1, top.getNome());
				stmt.execute();
			} finally {
				stmt.close();
			}
		}
		cache.put(top.getNome(), top);
	}

	public List<Topico> selectTodos(Connection con) throws SQLException {
		PreparedStatement stmt = con
				.prepareStatement("SELECT NOME FROM TOPICO");
		ResultSet rs = stmt.executeQuery();
		List<Topico> topicos = new ArrayList<Topico>();
		try {
			while (rs.next()) {
				Topico top = new Topico();
				top.setNome(rs.getString("NOME"));
				cache.put(top.getNome(), top);
				topicos.add(top);
			}
		} finally {
			rs.close();
			stmt.close();
		}
		return topicos;
	}

}
