package conferenciaBDAlunos;

import java.sql.Date;
import java.util.Calendar;
import java.util.List;

import junit.framework.TestCase;
import junit.framework.TestSuite;

public class TesteRevisaoConferencia extends TestCase {

	Conferencia conf;

	public TesteRevisaoConferencia() {
		conf = Conferencia.getInstance();
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	public void testCasoUso1() {
		String[] nomesTopicos = { "Modelagem", "Padroes de Projeto",
				"Projeto OO", "Gerenciamento de Projeto", "Testes", "Metricas",
				"Qualidade", "Refactoring" };
		Calendar cal = Calendar.getInstance();
		cal.set(2004, Calendar.DECEMBER, 8);
		Date dataAtual = new Date(cal.getTime().getTime());
		cal.set(2004, Calendar.JUNE, 8);
		Date dataInicioSubmissao = new Date(cal.getTime().getTime());
		cal.set(2004, Calendar.JULY, 8);
		Date dataTerminoSubmissao = new Date(cal.getTime().getTime());
		cal.set(2004, Calendar.SEPTEMBER, 8);
		Date dataTerminoRevisao = new Date(cal.getTime().getTime());
		conf.iniciaConferencia("SBES", "Florianopolis", dataAtual, 3,
				dataInicioSubmissao, dataTerminoSubmissao, dataTerminoRevisao,
				nomesTopicos);
		conf.defineChefe("Jose Silva", "silva@inf.ufsc.br", "silva");
		assertEquals(conf.getNome(), "SBES");
		assertEquals(conf.getLocal(), "Florianopolis");
		assertEquals(conf.getData(), dataAtual);
		assertEquals(conf.getDataInicioSubmissao(), dataInicioSubmissao);
		assertEquals(conf.getDataTerminoRevisao(), dataTerminoRevisao);
		assertEquals(conf.getDataTerminoSubmissao(), dataTerminoSubmissao);
		assertEquals(conf.getNumMaxArtigos(), 3);
	}

	public void testCasoUso2() {
		// Deveria conferir a data de inicio de submissao
		String[] nomesTopicosAvaliador1 = { "Modelagem", "Qualidade",
				"Refactoring" };
		conf.insereAvaliador("Avaliador1", "avalia1", "avalia1",
				"avalia1@inf.ufsc.br", "UFSC", nomesTopicosAvaliador1);
		String[] nomesTopicosAvaliador2 = { "Modelagem", "Padroes de Projeto",
				"Projeto OO", "Gerenciamento de Projeto", "Testes", "Metricas",
				"Qualidade" };
		conf.insereAvaliador("Avaliador2", "avalia2", "avalia2",
				"avalia2@inf.ufsc.br", "UFSC", nomesTopicosAvaliador2);
		String[] nomesTopicosAvaliador3 = { "Modelagem", "Padroes de Projeto",
				"Projeto OO", "Refactoring" };
		conf.insereAvaliador("Avaliador3", "avalia3", "avalia3",
				"avalia3@inf.ufsc.br", "UFSC", nomesTopicosAvaliador3);
		List<Avaliador> lista = conf.selecionaAvaliadores();
		assertEquals(lista.get(0).getNome(), "Avaliador1");
		assertEquals(lista.get(1).getNome(), "Avaliador2");
		assertEquals(lista.get(2).getNome(), "Avaliador3");
		assertEquals(lista.size(), 3);
	}

	public void testCasoUso3() {
		conf.submeteArtigo("Artigo1", "resumo...", "arquivo1",
				new String[] { "Autor1" }, new String[] { "UFRGS" },
				new String[] { "Padroes de Projeto", "Projeto OO" });
		conf.submeteArtigo("Artigo2", "resumo...", "arquivo2", new String[] {
				"Autor1", "Autor2" }, new String[] { "UFRGS", "UFSCar" },
				new String[] { "Padroes de Projeto", "Projeto OO" });
		conf.submeteArtigo("Artigo3", "resumo...", "arquivo3",
				new String[] { "Autor3" }, new String[] { "UFRGS" },
				new String[] { "Modelagem", "Projeto OO" });
		conf.submeteArtigo("Artigo4", "resumo...", "arquivo4", new String[] {
				"Autor3", "Autor4" }, new String[] { "UFSC", "UFRGS" },
				new String[] { "Projeto OO", "Refactoring" });
		List<Artigo> lista = conf.selecionaArtigos();
		assertEquals(lista.get(0).getNome(), "Artigo1");
		assertEquals(lista.get(1).getNome(), "Artigo2");
		assertEquals(lista.get(2).getNome(), "Artigo3");
		assertEquals(lista.get(3).getNome(), "Artigo4");
		assertEquals(lista.size(), 4);

	}

	public void testCasoUso4() {
		List<Artigo> artigosSemRevisao = conf.selecionaArtigosSemRevisao();
		assertEquals(artigosSemRevisao.get(0).getNome(), "Artigo1");
		assertEquals(artigosSemRevisao.get(1).getNome(), "Artigo2");
		assertEquals(artigosSemRevisao.get(2).getNome(), "Artigo3");
		assertEquals(artigosSemRevisao.get(3).getNome(), "Artigo4");

		List<Avaliador> avaliadores = conf
				.selecionaAvaliadoresSemConflito("Artigo1");
		assertEquals(avaliadores.get(0).getNome(), "Avaliador1");
		assertEquals(avaliadores.get(1).getNome(), "Avaliador2");
		assertEquals(avaliadores.get(2).getNome(), "Avaliador3");

		conf.defineConflito("Artigo1", "Avaliador1");
		avaliadores = conf.selecionaAvaliadoresSemConflito("Artigo1");
		assertEquals(avaliadores.get(0).getNome(), "Avaliador2");
		assertEquals(avaliadores.get(1).getNome(), "Avaliador3");
		assertEquals(avaliadores.size(), 2);

		artigosSemRevisao = conf.selecionaArtigosSemRevisao();
		assertEquals(artigosSemRevisao.get(0).getNome(), "Artigo1");
		assertEquals(artigosSemRevisao.get(1).getNome(), "Artigo2");
		assertEquals(artigosSemRevisao.get(2).getNome(), "Artigo3");
		assertEquals(artigosSemRevisao.get(3).getNome(), "Artigo4");

		avaliadores = conf.selecionaAvaliadoresSemConflito("Artigo4");
		assertEquals(avaliadores.get(0).getNome(), "Avaliador1");
		assertEquals(avaliadores.get(1).getNome(), "Avaliador2");
		assertEquals(avaliadores.get(2).getNome(), "Avaliador3");
		assertEquals(avaliadores.size(), 3);

		conf.defineConflito("Artigo4", "Avaliador1");
		avaliadores = conf.selecionaAvaliadoresSemConflito("Artigo4");
		assertEquals(avaliadores.get(0).getNome(), "Avaliador2");
		assertEquals(avaliadores.get(1).getNome(), "Avaliador3");
		assertEquals(avaliadores.size(), 2);
	}

	public void testCasoUso5() {
		List<Revisao> revisoes = conf.selecionaArtigo("Artigo1").getRevisoes();
		assertEquals(revisoes.size(), 0);
		conf.distribuiArtigos();
		// Artigo1 - Avaliador2
		// Artigo1 - Avaliador3

		// Artigo2 - Avaliador1
		// Artigo2 - Avaliador2
		// Artigo2 - Avaliador3

		// Artigo3 - Avaliador1
		// Artigo3 - Avaliador2
		// Artigo3 - Avaliador3

		// Artigo4 - Avaliador2
		// Artigo4 - Avaliador3

		revisoes = conf.selecionaArtigo("Artigo1").getRevisoes();
		assertEquals(revisoes.size(), 2);
		assertEquals(revisoes.get(0).getAvaliador().getNome(), "Avaliador2");
		assertEquals(revisoes.get(1).getAvaliador().getNome(), "Avaliador3");

		revisoes = conf.selecionaArtigo("Artigo2").getRevisoes();
		assertEquals(revisoes.size(), 3);
		assertEquals(revisoes.get(0).getAvaliador().getNome(), "Avaliador1");
		assertEquals(revisoes.get(1).getAvaliador().getNome(), "Avaliador2");
		assertEquals(revisoes.get(2).getAvaliador().getNome(), "Avaliador3");

		revisoes = conf.selecionaArtigo("Artigo3").getRevisoes();
		assertEquals(revisoes.size(), 3);
		assertEquals(revisoes.get(0).getAvaliador().getNome(), "Avaliador1");
		assertEquals(revisoes.get(1).getAvaliador().getNome(), "Avaliador2");
		assertEquals(revisoes.get(2).getAvaliador().getNome(), "Avaliador3");

		revisoes = conf.selecionaArtigo("Artigo4").getRevisoes();
		assertEquals(revisoes.size(), 2);
		assertEquals(revisoes.get(0).getAvaliador().getNome(), "Avaliador2");
		assertEquals(revisoes.get(1).getAvaliador().getNome(), "Avaliador3");
	}

	public void testCasoUso6() {
		conf.forneceRevisao("Artigo1", "Avaliador2", 4, 4, 4, 4, "coment",
				"coment");
		conf.forneceRevisao("Artigo1", "Avaliador3", 2, 2, 2, 2, "coment",
				"coment");
		List<Revisao> revisoesArtigo1 = conf.selecionaArtigo("Artigo1")
				.getRevisoes();
		assertEquals(revisoesArtigo1.get(0).getNotaFinal(), 4.0);
		assertEquals(revisoesArtigo1.get(1).getNotaFinal(), 2.0);

		conf.forneceRevisao("Artigo2", "Avaliador1", 2, 2, 2, 2, "coment",
				"coment");
		conf.forneceRevisao("Artigo2", "Avaliador2", 2, 2, 2, 2, "coment",
				"coment");
		conf.forneceRevisao("Artigo2", "Avaliador3", 3, 3, 3, 3, "coment",
				"coment");
		List<Revisao> revisoesArtigo2 = conf.selecionaArtigo("Artigo2")
				.getRevisoes();
		assertEquals(revisoesArtigo2.get(0).getNotaFinal(), 2.0);
		assertEquals(revisoesArtigo2.get(1).getNotaFinal(), 2.0);
		assertEquals(revisoesArtigo2.get(2).getNotaFinal(), 3.0);

		conf.forneceRevisao("Artigo3", "Avaliador1", 2, 2, 2, 2, "coment",
				"coment");
		conf.forneceRevisao("Artigo3", "Avaliador2", 2, 2, 2, 2, "coment",
				"coment");
		conf.forneceRevisao("Artigo3", "Avaliador3", 2, 2, 2, 2, "coment",
				"coment");
		List<Revisao> revisoesArtigo3 = conf.selecionaArtigo("Artigo3")
				.getRevisoes();
		assertEquals(revisoesArtigo3.get(0).getNotaFinal(), 2.0);
		assertEquals(revisoesArtigo3.get(1).getNotaFinal(), 2.0);
		assertEquals(revisoesArtigo3.get(2).getNotaFinal(), 2.0);

		conf.forneceRevisao("Artigo4", "Avaliador2", 5, 5, 5, 5, "coment",
				"coment");
		conf.forneceRevisao("Artigo4", "Avaliador3", 2, 2, 2, 2, "coment",
				"coment");
		List<Revisao> revisoesArtigo4 = conf.selecionaArtigo("Artigo4")
				.getRevisoes();
		assertEquals(revisoesArtigo4.get(0).getNotaFinal(), 5.0);
		assertEquals(revisoesArtigo4.get(1).getNotaFinal(), 2.0);
	}

	public void testCasoUso7() {
		List<Artigo> artigosAvaliados = conf.selecionaArtigosComNotas();
		assertEquals(artigosAvaliados.size(), 4);
		assertEquals(artigosAvaliados.get(0).getNotaFinal(), 3.0);
		assertEquals((int) (artigosAvaliados.get(1).getNotaFinal() * 10), 23);
		assertEquals(artigosAvaliados.get(2).getNotaFinal(), 2.0);
		assertEquals((int) (artigosAvaliados.get(3).getNotaFinal() * 10), 35);
		conf.defineArtigosAceitos(new String[] { "Artigo1", "Artigo2",
				"Artigo4" });
		assertEquals(conf.selecionaArtigo("Artigo1").getStatus(), 1);
		assertEquals(conf.selecionaArtigo("Artigo2").getStatus(), 1);
		assertEquals(conf.selecionaArtigo("Artigo3").getStatus(), 0);
		assertEquals(conf.selecionaArtigo("Artigo4").getStatus(), 1);
		conf.modificaStatusArtigo("Artigo2");
		conf.modificaStatusArtigo("Artigo3");
		assertEquals(conf.selecionaArtigo("Artigo2").getStatus(), 0);
		assertEquals(conf.selecionaArtigo("Artigo3").getStatus(), 1);
	}

	public void testFinal() {
		conf.termina();
		System.out.println("fechando conexao!");
	}

	public static void main(String[] args) {
		TestSuite suite = new TestSuite(TesteRevisaoConferencia.class);
		junit.textui.TestRunner.run(suite);
	}

}
